﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Controls.Charts;
using System.Windows.Controls.Primitives;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void XamDataChart_SeriesCursorMouseMove(object sender, ChartCursorEventArgs e)
        {

        }
    }
    public class ChartBehaviors : DependencyObject
    {
        public static readonly DependencyProperty CursorTooltipProperty =
            DependencyProperty.RegisterAttached("CursorTooltip",
            typeof(CursorTooltipBehavior), typeof(ChartBehaviors),
            new PropertyMetadata(null,
                (o, e) => CursorTooltipChanged(
                    o as XamDataChart,
                    e.OldValue as CursorTooltipBehavior,
                    e.NewValue as CursorTooltipBehavior)));

        public static CursorTooltipBehavior GetCursorTooltip(
            DependencyObject target)
        {
            return target.GetValue(CursorTooltipProperty)
                as CursorTooltipBehavior;
        }

        public static void SetCursorTooltip(
            DependencyObject target, CursorTooltipBehavior behavior)
        {
            target.SetValue(CursorTooltipProperty, behavior);
        }

        private static void CursorTooltipChanged(
            XamDataChart chart,
            CursorTooltipBehavior oldValue,
            CursorTooltipBehavior newValue)
        {
            if (chart == null)
            {
                return;
            }
            if (oldValue != null)
            {
                oldValue.OnDetach(chart);
            }
            if (newValue != null)
            {
                newValue.OnAttach(chart);
            }
        }
    }

    public class SeriesItemInfo
        : INotifyPropertyChanged
    {
        private Series _series;
        public Series Series
        {
            get { return _series; }
            set
            {
                _series = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged(
                        this,
                        new PropertyChangedEventArgs("Series"));
                }
            }
        }

        private object _item;
        public object Item
        {
            get { return _item; }
            set
            {
                _item = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged(
                        this,
                        new PropertyChangedEventArgs("Item"));
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }

    public class SeriesItemInfoCollection
        : ObservableCollection<SeriesItemInfo>
    {
        public void UpdateSeriesItem(
            Series series,
            object item)
        {
            bool found = false;
            var indexes = from curr in this
                          where curr.Series == series
                          select this.IndexOf(curr);
            foreach (int index in indexes)
            {
                found = true;
                this[index].Item = item;
            }

            if (!found)
            {
                this.Add(
                    new SeriesItemInfo()
                    {
                        Series = series,
                        Item = item
                    });
            }
        }
    }

    public class CursorTooltipBehavior
    {
        private bool _isOverChart = false;
        private Popup _popup = new Popup();
        
        private ContentControl _content =
            new ContentControl();
        private Panel _container;
        private XamDataChart _owner = null;

        private DataTemplate _tooltipTemplate;

        private SeriesItemInfoCollection _items
            = new SeriesItemInfoCollection();

        public DataTemplate TooltipTemplate
        {
            get { return _tooltipTemplate; }
            set
            {
                _tooltipTemplate = value;
                _content.ContentTemplate = _tooltipTemplate;
            }
        }

        protected bool IsOverChart
        {
            get { return _isOverChart; }
            set
            {
                bool last = _isOverChart;
                _isOverChart = value;
                if (_isOverChart && !last)
                {
                    ShowPopup();
                }
                if (!_isOverChart && last)
                {
                    HidePopup();
                }
            }
        }

        private void HidePopup()
        {
            _popup.IsOpen = false;
        }

        private void ShowPopup()
        {

            //HERE IS THE CHNAGE I MADE
            _popup.Placement = PlacementMode.RelativePoint;



            _popup.IsOpen = true;
        }

        public void OnAttach(XamDataChart chart)
        {
            if (_owner != null)
            {
                OnDetach(_owner);
            }
            chart.MouseLeave += Chart_MouseLeave;
            chart.MouseMove += Chart_MouseMove;
            chart.SeriesCursorMouseMove +=
                Chart_SeriesCursorMouseMove;

            _popup.IsOpen = false;
            _popup.Child = _content;
            _content.ContentTemplate = TooltipTemplate;
            _content.Content = _items;
            if (chart.Parent != null && chart.Parent is Panel)
            {
                _container = chart.Parent as Panel;
                _container.Children.Add(_popup);
            }
        }

        public void OnDetach(XamDataChart chart)
        {
            if (_owner != chart)
            {
                return;
            }
            chart.MouseLeave -= Chart_MouseLeave;
            chart.MouseMove -= Chart_MouseMove;
            chart.SeriesCursorMouseMove -=
                Chart_SeriesCursorMouseMove;
            IsOverChart = false;
            _items.Clear();
            _owner = null;
        }

        void Chart_MouseMove(object sender, MouseEventArgs e)
        {

            //I ADDED THE ROW BELOW, TOO
            _popup.PlacementTarget = sender as XamDataChart;


            SetPopupOffsets(e.GetPosition(_owner));
            IsOverChart = true;
        }

        private void SetPopupOffsets(Point point)
        {
            _popup.VerticalOffset = point.Y + 10;
            _popup.HorizontalOffset = point.X + 10;
        }

        void Chart_SeriesCursorMouseMove(
            object sender, ChartCursorEventArgs e)
        {
            if (e.Series != null && e.Item != null)
            {
                _items.UpdateSeriesItem(e.Series, e.Item);
            }
        }

        void Chart_MouseLeave(
            object sender, MouseEventArgs e)
        {
            IsOverChart = false;
        }
    }

    public class TestData : ObservableCollection<Data>
    {
        public TestData()
        {
                for (int i = 1; i < 10; i++)
                {
                    this.Add(
                        new Data()
                        {
                            Date = DateTime.Now.AddHours(i),
                            Value = i
                        });
                }
        }
    }

    public class TestData2 : ObservableCollection<Data>
    {
        public TestData2()
        {
            for (int i = 1; i < 20; i+=2)
            {
                this.Add(
                    new Data()
                    {
                        Date = DateTime.Now.AddHours(i),
                        Value = i
                    });
            }
        }
    }

    public class Data
    {
        public DateTime Date { get; set; }
        public int Value { get; set; }
    }
}
